
# q-expansion of newform 162.4.a.h, downloaded from the LMFDB on 13 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 162
weight = 4
poly_data = [-3, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[0, 3], 1]]

hecke_ring_character_values = None
aps_data = [[2, 0], [0, 0], [6, 1], [8, 2], [18, -8], [5, -14], [60, 11], [-4, 22], [90, -4], [162, 7], [-124, -36], [-217, 2], [96, -44], [-304, -6], [-192, -36], [-204, 76], [-504, 32], [371, -18], [-52, 138], [-570, 8], [425, -96], [-220, -50], [132, -60], [384, 89], [-382, -56], [-708, 176], [-1060, -60], [84, -260], [821, 174], [-384, -261], [-412, 174], [402, 292], [1620, 193], [764, -268], [2070, -289], [-952, 252], [551, -246], [-64, -64], [-438, 696], [1950, 39], [-3264, -236], [2114, -136], [606, 32], [1595, 396], [1842, 155], [140, 256], [-1696, 622], [-1564, -18], [294, -348], [1649, 510], [-2472, 733], [-3948, -564], [305, 236], [1722, -104], [4584, -39], [-2904, 20], [-1878, 901], [-5740, -114], [-70, 192], [2664, -293], [4484, -168], [2610, -55], [4412, 492], [6756, -80], [539, 356], [-1086, -919], [860, 790], [602, -712], [2370, 1092], [1682, -544], [7344, 88], [-594, 180], [-5056, 996], [-1714, -512], [116, 1700], [6234, -1160], [-7620, 1076], [-5929, -138], [492, -95], [-3439, -532], [-8628, -216], [-5707, -838], [-1380, -1036], [-313, -1980], [2396, -2880], [-840, -1812], [-4392, 60], [-4615, 468], [-1260, 868], [2696, 3136], [588, 1216], [-3594, 2112], [-796, -1532], [10674, 224], [15032, -1242], [3996, -1464], [60, 1732], [-2712, -1948], [13016, -1146], [-16543, -982], [9380, -168], [7350, -1537], [10632, 564], [-19428, -325], [-352, -2728], [-14965, 72], [16554, 344], [22992, 1037], [-540, -4492], [9263, 1416], [-2080, 98], [5582, -2184], [9888, -563], [6464, -732], [-21160, 140], [21636, 853], [16868, 828], [5922, 456], [15228, 1820], [-6738, -4468], [-4345, 42], [-19741, -1268], [10740, -796], [-5688, 3540], [-1420, -2754], [-6270, 1081], [-9535, -2954], [12906, 1620], [15608, -1458], [3890, 4728], [-12004, 192], [6960, -444], [-24040, 1286], [2738, -504], [588, -1815], [-7429, 1948], [-486, -3393], [11576, 3882], [-3714, -4725], [-18012, -4723], [11432, -3528], [-3606, 4997], [3452, -512], [9870, 2528], [-17374, 376], [4326, 8076], [34886, -1344], [-13260, 3813], [10124, 4760], [-5502, -1036], [-17137, -2074], [21648, -2108], [2372, 172], [27330, -3452], [10400, 604], [9186, 5508], [3332, 5262], [34836, -1293], [24335, 1448], [-6498, 815], [12282, -3708], [29196, -531], [19136, -3702], [23562, -4404], [19080, -5548], [18516, -4240], [-5632, 966], [2099, 4014]]
